import { Map } from 'immutable';

export const updateEntities = (update, entities = new Map(), transformed = new Map()) => {
  // If update, not supplied, then no change to entities should be made
  return Object.keys(update || {}).reduce((accum, key) => {
    return accum.set(key, update[key](entities.get(key), transformed.get(key)));
  }, new Map());
};

export const optimisticUpdateEntities = (optimisticUpdate, entities = new Map()) => {
  return Object.keys(optimisticUpdate).reduce((accum, key) => {
    if (optimisticUpdate[key]) {
      return accum.set(key, optimisticUpdate[key](entities.get(key)));
    }
    return accum.set(key, entities.get(key));
  }, new Map());
};

export const rollbackEntities = (rollback = {}, initialEntities, entities) => {
  return Object.keys(initialEntities.toObject()).reduce((accum, key) => {
    if (rollback[key]) {
      return accum.set(key, rollback[key](initialEntities.getIn([key]), entities.getIn([key])));
    } else {
      // Default to just reverting to the initial state for that
      // entity (before the optimistic update)
      return accum.set(key, initialEntities.getIn([key]));
    }
  }, new Map());
};
